# Elevate
# Copyright 2007 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

import pygame
from screen import *
from constants import *

WIDTH_MULTIPLE = 1.5

MAX_VELOCITY = 5

ELEV_BASE_LINEWIDTH = 6.0

ELEV_CAPACITY = 8

ELEV_ANGER_STOP_PROB = 20

def getUnscaledElevatorWidth():
    return WIDTH_MULTIPLE*getUnscaledFloorHeight()

def drawShaft(surf):
    x1 = int(getScaling()*getUnscaledElevatorWidth())+calculateFloorX()
    pygame.draw.line(surf,Color('Black'),(x1,0),(x1,surf.get_height()),4)
    
    x1 = calculateFloorX()
    pygame.draw.line(surf,Color('Black'),(x1,0),(x1,int(getScaling()*TOP_BUFFER)),4)
    pygame.draw.line(surf,Color('Black'),(x1,surf.get_height()),(x1,surf.get_height()-int(getScaling()*BOTTOM_BUFFER)),4)

class Elevator(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)

        self.velocity = 0
        
        w = int(ELEV_WIDTH_MULTIPLE*getFloorHeight())
        h = getFloorHeight()
        
        self.rect = Rect(calculateFloorX(),calculateFloorY(1),w,h)
        self.rect.bottom = calculateFloorY(1)
        self.rect.left = calculateFloorX()
        self.color = Color('Black')
        
        self.passengers = []
        
        self.loading = False
        
    def moveUp(self):
        self.velocity = self.velocity - 1
        self.safeVelocity()
        self.loading = False
        
    def moveDown(self):
        self.velocity = self.velocity + 1
        self.safeVelocity()
        self.loading = False
            
    def safeVelocity(self):
        if self.velocity > MAX_VELOCITY:
            self.velocity = MAX_VELOCITY
        if self.velocity < (-1*MAX_VELOCITY):
            self.velocity = (-1*MAX_VELOCITY)
    
    def fullStop(self):
        self.velocity = 0
    
    def updateRect(self):
        if self.rect.height != getFloorHeight():
            scalefactor = float(getFloorHeight())/float(self.rect.height)
            self.rect.width = int(self.rect.width*scalefactor)
            self.rect.height = int(self.rect.height*scalefactor)
            self.rect.left = calculateFloorX()
    
    def createSurface(self):
        self.image = pygame.Surface((self.rect.width,self.rect.height))
        self.image.convert()
        self.image.set_colorkey(Color('White'))
        self.image.fill(Color('White'))
    
    def move(self, screen):
        if self.velocity < 0 and self.rect.top > 0:
            self.rect.bottom = self.rect.bottom + self.velocity
        elif self.velocity > 0 and self.rect.bottom < screen.get_height():
            self.rect.bottom = self.rect.bottom + self.velocity
        else:
            self.velocity = 0

    def getPassengers(self):
        return self.passengers

    def getAbsVelocity(self):
        return abs(self.velocity)
    
    def getVelocity(self):
        return self.velocity
    
    def getDirection(self):
        if self.velocity == 0:
            return 'Idle'
        elif self.velocity > 0:
            return 'Down'
        else:
            return 'Up'

    def getElevatorY(self):
        return self.rect.bottom
    
    def getElevatorX(self,passenger):
        x = (self.rect.width-2)/(ELEV_CAPACITY+1)
        indexof = 0
        try:
            indexof = self.passengers.index(passenger)
        except:
            indexof = 0
        
        x = indexof*x + self.rect.left
        return x

    def getCurrentFloor(self):
        my_y = self.getElevatorY()
        all_ys = getAllFloorY()
        min_diff = 32768
        
        for i in range(0,NUMBER_FLOORS):
            diff = abs(my_y-all_ys[i])
            if(diff < min_diff):
                min_diff = diff
            else:
                i = i - 1
                break
        i = i + 1
        
        return i

    def getUnloaded(self):
        unloaded = []
        if self.velocity != 0:
            return unloaded
        
        fl = self.getCurrentFloor()
        dy = abs(self.getElevatorY() - calculateFloorY(fl))
        for x in self.passengers:
            if x.getTgt() == fl:
                unloaded.append(x)
                if dy > getScaling()*ELEV_ANGER_STOP_PROB and ANGRY_EXITS:
                    x.setAngry()
		
        for x in unloaded:
            self.passengers.remove(x)
		
        return unloaded

    def load(self, fl_pass):
        # Passengers should be furious if you open the doors
        # while moving
        if self.velocity != 0:
            for x in self.passengers:
                x.setAngry()
				
        # Otherwise
        else:
            self.loading = True
            while len(self.passengers) < ELEV_CAPACITY:
                if len(fl_pass) > 0:
                    self.passengers.append(fl_pass[0])
                    fl_pass[0].setRiding()
                    fl_pass.remove(fl_pass[0])
                else:
                    break
                

    
    def getTargets(self):
        targets = []
        for x in self.passengers:
            targets.append(x.getTgt())
        return targets

    def update(self):
        self.updateRect()
        
        # Create the surface
        self.createSurface()
        
        if self.loading:
            pygame.draw.rect(self.image,Color('Grey'),Rect(0,0,self.rect.width/4,self.rect.height),0)
            pygame.draw.rect(self.image,Color('Grey'),Rect(3*self.rect.width/4,0,self.rect.width/4,self.rect.height),0)
        else:
            pygame.draw.rect(self.image,Color('Grey'),Rect(0,0,self.rect.width,self.rect.height),0)
        
        # Draw elevator
        lw = int(ELEV_BASE_LINEWIDTH*getScaling())
        pygame.draw.rect(self.image,self.color,Rect(0,0,self.rect.width,self.rect.height),lw)
        
        
