# Elevate
# Copyright 2007 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

import pygame
import random

from passenger import *

from screen import *

from constants import *

CREATE_TIMING = 60

WAIT_SPACE_BUFFER = 10

FLOOR_FONT = 'CAB.TTF'

MAX_FLOOR_PASSENGERS = 10

def drawFloors(screen,floors):
    for x in floors:
        x.draw(screen)

def createAndDrawFloors(screen):
    floors = []
    for x in range(1,NUMBER_FLOORS+1):
        one_floor = Floor(x,NUMBER_FLOORS)
        floors.append(one_floor)
    drawFloors(screen,floors)
    return floors

class Floor:

    base_line_width = 2.0
    color = Color('black')
    def __init__(self, number, total):
        self.number = number
        self.total_floors = total
        self.passengers = []
        self.number = number
        self.create_prob = 0.001
        self.total_floors = 10
        self.font = pygame.font.Font(filepath(FLOOR_FONT),25)
        
        self.allowcreate = True
        
        if self.number == 1:
            self.create_prob = 0.01

    def setAllowCreate(self,value):
        self.allowcreate = value

    def addPassenger(self,passenger):
        self.passengers.append(passenger)
    
    def addPassengers(self,passengers):
    	for x in passengers:
    		self.passengers.append(x)

    def removePassenger(self,passenger):
        self.passengers.remove(passenger)

    def createPassenger(self):
        # Randomly generate a floor target
        tgt = random.randint(1,self.total_floors)
        while tgt == self.number:
            tgt = random.randint(1,self.total_floors)

        # Randomly generate a patience
        pat = random.randint(0,MAX_START_PATIENCE)
        patdot = random.random()*(MAX_PATIENCE_RATE-MIN_PATIENCE_RATE)+MIN_PATIENCE_RATE

        # Now create
        toadd = Passenger(self.number,tgt,pat,patdot)

        # Add to the array
        self.addPassenger(toadd)

    def forceNewPassenger(self):
        self.createPassenger()

    def newPassengers(self):
        if self.allowcreate:
            if random.random() < self.create_prob:
                # print 'Creating passenger - Floor: ',self.number
                if self.getNumberWaitingAndEnteringPassengers()<MAX_FLOOR_PASSENGERS:
                    self.createPassenger()

    def getWaitingPassengers(self):
        waiting = []
        for x in self.passengers:
            if x.isWaiting():
                waiting.append(x)
        return waiting

    def setProbability(self, value):
        self.create_prob = value

    def getNumberWaitingAndEnteringPassengers(self):
        return len(self.getWaitingAndEnteringPassengers())

    def getWaitingAndEnteringPassengers(self):
        waiting = []
        for x in self.passengers:
            if x.isWaiting() or x.isEntering():
                waiting.append(x)
        return waiting

    def getPassengers(self):
        return self.passengers

    def getY(self):
        return calculateFloorY(self.number)

    def update(self):
        # First generate any new passengers
        self.newPassengers()

    def getWaitX(self):
        max_x = calculateFloorX() - int(getScaling()*WAIT_SPACE_BUFFER)
        # For now, return this
        return max_x
    
    def getWaitX(self,p):
        max_x = self.getAbsoluteWaitX()
        waiters = self.getWaitingAndEnteringPassengers()
        
        try:
            idx = waiters.index(p)
        except:
            idx = 0

        return max_x - int(1.1*(idx*p.rect.width))
    
    def getAbsoluteWaitX(self):
        return calculateFloorX() - int(getScaling()*WAIT_SPACE_BUFFER)
    
    def removePassenger(self,passenger):          
        self.passengers.remove(passenger)

    def draw(self,surface):
        x = calculateFloorX()
        lw = int(self.base_line_width*getScaling())
        firsty = self.getY()
        secondy = calculateFloorY(self.number+1)
        
        # Draw two lines
        pygame.draw.line(surface, self.color, (0,firsty), (x,firsty), lw)
        pygame.draw.line(surface, self.color, (0,secondy), (x,secondy), lw)
        
        # Cap the end with dots
        for yd in range(secondy,firsty-2,4):
            pygame.draw.line(surface, self.color, (x,yd), (x,yd+2), lw)

        fnum = self.font.render(str(self.number),1,(128,128,128))
        surface.blit(fnum,(int(0.8*x),(firsty+3*secondy)/4))

