# Elevate
# Copyright 2007 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
import sys
import data
import pygame
from pygame import *
from floor import *
from passenger import *
from elevator import *
from screen import *
from panel import *
from constants import *
from menu import *
from level import *
from loser import *
from manual import *

def main():
    
    developer = False
    
    pygame.init()
    
    # Sound init
    sound = True
    if pygame.mixer and not pygame.mixer.get_init():
        print 'Warning: no sound...'
        pygame.mixer = None
        sound = False
    else:
        pygame.mixer.init()
    
    
    screen = pygame.display.set_mode((BASE_WIDTH,BASE_HEIGHT))
    pygame.display.set_caption(TITLE)
    
    # Load sounds
    print 'Loading sounds...'
    elev_sound = None
    music_sound = None
    if sound:
        elev_sound = pygame.mixer.Sound(filepath('floor.wav'))
        music_sound = pygame.mixer.Sound(filepath('music.ogg'))    
    
    # init bkgd
    print 'Creating Background...'
    bkgd = fillBkgd(screen)
    
    # draw floors
    print 'Creating floors....'
    floors = createAndDrawFloors(bkgd)
    
    # Load level data
    levels = LevelHandler()
    levels.load()
    
    # Draw shaft
    drawShaft(bkgd)
    
    screen.blit(bkgd,(0,0))
    
    # and display
    print 'Flipping...'
    pygame.display.flip()
    
    # Timing...
    print 'Starting clock...'
    clock = pygame.time.Clock()
    
    # Initializing color scaling
    print 'Generating RGB scales...'
    preComputeRGBScaling() 

    # Create the elevator
    elevator = Elevator()
    
    # Score and status panel
    panel = Panel(screen,screen.get_width()-elevator.rect.width-calculateFloorX())

    # menu sprite
    menu = Menu()
    manual = Manual()
    
    # Loser info sprite
    loser = None
    
    # Sprite renderers
    pass_sprites = pygame.sprite.RenderUpdates()
    elev_sprite = pygame.sprite.RenderUpdates(elevator)
    panel_sprite = pygame.sprite.RenderUpdates(panel)
    
    overlay_sprite = pygame.sprite.RenderUpdates()

	# Scoring
    delivered = 0
    complaints = 0
    
    print 'Entering main loop...'
    menu_mode = True
    manual_mode = False
    loser_mode = False
    menu_process = False
    manual_process = False
    loser_process = False
    run_game = True
    load_elev = False
    last_creation_time = 0
    last_floor = 1
    
    # Start the music
    music_sound.play(-1)
    music_playing = True
    
    while run_game:
        clock.tick(60)
        current_time = pygame.time.get_ticks()
        
        pass_sprites.clear(screen,bkgd)
        elev_sprite.clear(screen,bkgd)
        panel_sprite.clear(screen,bkgd)
        overlay_sprite.clear(screen,bkgd)
        
        directions = [False, False]

        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                run_game = False
            elif event.type == KEYDOWN and menu_mode:
                if event.key == K_UP:
                    menu.moveUp()
                elif event.key == K_DOWN:
                    menu.moveDown()
                elif event.key == K_SPACE or event.key == K_RETURN:
                    menu_process = True
            elif event.type == KEYDOWN and manual_mode:
                if event.key == K_SPACE or event.key == K_RETURN or event.key == K_ESCAPE:
                    manual_process = True
            elif event.type == KEYDOWN and loser_mode:
                if event.key == K_SPACE or event.key == K_RETURN or event.key == K_ESCAPE:
                    loser_process = True
            elif event.type == KEYDOWN:
                if event.key == K_UP:
                    directions[UP] = True
                    load_elev = False
                elif event.key == K_DOWN:
                    directions[DOWN] = True
                    load_elev = False
                elif event.key == K_SPACE:
                    load_elev = True
                elif event.key == K_ESCAPE or event.key == K_q:
                    menu.useStart(False)
                    menu_mode = True
                elif event.key == K_a and developer:
                    delivered = delivered + 1
                elif event.key == K_m and sound:
                    if music_playing:
                        music_sound.stop()
                        music_playing = False
                    else:
                        music_sound.play(-1)
                        music_playing = True
        
        # Move the elevator
        if directions[UP] == True:
            elevator.moveUp()
        elif directions[DOWN] == True:
            elevator.moveDown()
        elevator.move(screen)
        
        # Game processing
        if menu_mode:
            # MENU MODE Code ----------------------------------------------------
            overlay_sprite.add(menu)
            if menu_process:
                if menu.getSelection() == 0:
                    menu_mode = False
                    overlay_sprite.remove(menu)
                    
                    if menu.isStart():
                        # Force an initial passenger
                        floors[0].forceNewPassenger()
            
                elif menu.getSelection() == 1:
                    menu_mode = False
                    manual_mode = True
                    overlay_sprite.remove(menu)
                    overlay_sprite.add(manual)
                elif menu.getSelection() == 2:
                    run_game = False
                menu_process = False
            overlay_sprite.update()
            dirty_sprites = overlay_sprite.draw(screen)
            # END MENU MODE Code ------------------------------------------------
        elif manual_mode:
            # MANUAL MODE Code --------------------------------------------------
            overlay_sprite.add(manual)
            if manual_process:
                overlay_sprite.remove(manual)
                overlay_sprite.add(menu)
                menu_mode = True
                manual_mode = False
                manual_process = False
                
            overlay_sprite.update()
            dirty_sprites = overlay_sprite.draw(screen)
            # END MANUAL MODE Code ----------------------------------------------
        elif loser_mode:
            # LOSER MODE Code ---------------------------------------------------
            if loser == None:
                loser = LoserPanel(delivered)
                loser.setMessage(levels.getGoodbye())
            overlay_sprite.add(loser)
            if loser_process:
                overlay_sprite.remove(loser)
                overlay_sprite.add(menu)
                menu_mode = True
                loser_mode = False
                loser_process = False
                menu.useStart(True)
                
            overlay_sprite.update()
            dirty_sprites = overlay_sprite.draw(screen)
            # END LOSER MODE Code -----------------------------------------------
        else:
            
            # GAME MODE Code ----------------------------------------------------
            efloor = elevator.getCurrentFloor()
            if efloor != last_floor and elev_sound != None:
                last_floor = efloor
                elev_sound.stop()
                elev_sound.play(0,0)
            
            pass_sprites.empty()
            
            # Load the elevator if necessary
            if load_elev:
                
                if SPACE_STOPS_IT:
                    elevator.fullStop()
                
                fl_idx = efloor - 1
            
                # First unload to generate space in the elevator
                outgoing = elevator.getUnloaded()
                for x in outgoing:
                    x.arrived(floors[fl_idx])
                    floors[fl_idx].addPassenger(x)
                delivered = delivered + len(outgoing)
            
                # Now load up new passengers
                elevator.load(floors[fl_idx].getWaitingPassengers())
            
            # Check if the floors can make passengers
            make_passengers = False
            if current_time - last_creation_time > CREATE_AFTER:
                last_creation_time = current_time
                make_passengers = True
            
            # Update the floors next
            for fl in floors:
                
                fl.setAllowCreate(make_passengers)
                fl.update()
                
                for p in fl.getPassengers():
                    if p.isDone():
                    
                        if p.isLeavingAngry():
                            complaints = complaints + 1
    
                        #print 'Removing...'
                        fl.removePassenger(p)
                
                # Get the current list of passengers
                pass_sprites.add(fl.getPassengers())
            
            # Add elevator passengers to the sprite list for update
            pass_sprites.add(elevator.getPassengers())
            
            # Check if there are any passengers at all
            if len(pass_sprites) == 0:
                floors[0].forceNewPassenger()
                pass_sprites.add(floors[0].getPassengers())
            
            # Check our level handler now
            levels.checkLevel(delivered)
            if levels.isChanged():
                updateFloors(levels,floors)
                panel.setLevelTitle(levels.getTitle())
                
            
            # Update passengers
            for x in pass_sprites:
                x.move(PEOPLE_VELOCITY,elevator,floors[x.getFloor()-1])
            
            # update the panel
            panel.setScore(delivered,complaints)
            panel.setElevator(str(efloor), elevator.getVelocity())
            panel.setTargets(elevator.getTargets())
            
            # Check our complaints
            if complaints > MAX_COMPLAINTS:
                loser_mode = True
            
            # Sprite updates
            pass_sprites.update()
            elev_sprite.update()
            panel_sprite.update()
        
            dirty_sprites = elev_sprite.draw(screen) + \
                            panel_sprite.draw(screen) + \
                            pass_sprites.draw(screen)
        
            # End GAME MODE Code ------------------------------------------------
            
        pygame.display.update(dirty_sprites)
        
    print 'Shutting down.'
    sys.exit()
