# Elevate
# Copyright 2007 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

import pygame
from data import *
from passenger import *
from screen import *
from constants import *

TITLE_FONT_NAME = "CAB.TTF"

class Menu(pygame.sprite.Sprite):
    options=['Start','Help','Quit']
    extra_op = 'Continue'
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        
        self.titlefont = pygame.font.Font(filepath(TITLE_FONT_NAME),64)
        self.itemfont = pygame.font.Font(filepath(TITLE_FONT_NAME),40)
        self.infofont = pygame.font.Font(filepath(TITLE_FONT_NAME),20)
        
        self.bkgdcolor = Color('White')
        self.fontcolor = Color('Black')
        self.selcolor = Color('Red')
        self.selected = 0
        # self.updateRect()
        self.rect = Rect(0,0,0,0)
        
        # Predraw menu images
        self.uimage = []
        self.simage = []
        self.holding_start = True
        self.predrawMenu()
        self.useStart(True)
        
        self.anim = Passenger(1,2,0,0)
        self.anim.setAngryWaiting()
        self.anim.setFootRate(5.0)
        self.anim.setFootSize(0.08)
        self.anim.setDrawTgtAndAnger(False)
    
    
    def predrawMenu(self):
        for s in range(0,len(self.options)):
            self.uimage.append(self.itemfont.render(self.options[s],True,self.fontcolor,self.bkgdcolor))
            self.simage.append(self.itemfont.render(self.options[s],True,self.selcolor,self.bkgdcolor))
        
    def useStart(self,value):
        if value and not self.holding_start:
            self.holding_start = True
            self.uimage[0] = self.itemfont.render(self.options[s],True,self.fontcolor,self.bkgdcolor)
            self.simage[0] = self.itemfont.render(self.options[s],True,self.selcolor,self.bkgdcolor)
            self.drawMenuItem(0,self.selected==0)
        elif not value and self.holding_start:
            self.holding_start = False
            self.uimage[0] = self.itemfont.render(self.extra_op,True,self.fontcolor,self.bkgdcolor)
            self.simage[0] = self.itemfont.render(self.extra_op,True,self.selcolor,self.bkgdcolor)
            self.drawMenuItem(0,self.selected==0)
    
    def isStart(self):
        return self.holding_start
    
    def drawMenuItem(self,which,selected):
        if selected:
            the_image = self.simage[which]
        else:
            the_image = self.uimage[which]
        self.image.blit(the_image,(self.rect.width/5,self.rect.height/3+int((0.1+float(which))*the_image.get_height())))
                
    def moveUp(self):
        self.drawMenuItem(self.selected,False)
        self.selected = max(self.selected-1,0)
        self.drawMenuItem(self.selected,True)
    
    def moveDown(self):
        self.drawMenuItem(self.selected,False)
        self.selected = min(self.selected+1,len(self.options)-1)
        self.drawMenuItem(self.selected,True)

    def getSelection(self):
        return self.selected

    def updateRect(self):
        self.rect = Rect(0,0,int(getScaling()*BASE_WIDTH),int(getScaling()*BASE_HEIGHT))

    def update(self):
        scale = getScaling()
        if self.rect.width != int(scale*BASE_WIDTH):
            self.updateRect()
            self.image = Surface((self.rect.width,self.rect.height))
            self.image.convert()
            self.image.fill(self.bkgdcolor)
            
            # Title
            gtitle = self.titlefont.render(TITLE,True,self.fontcolor)
            self.image.blit(gtitle,(self.rect.width/15,self.rect.height/15))
            
            # Info1
            ginfo0 = self.infofont.render(INFO0,True,self.fontcolor)
            ginfo1 = self.infofont.render(INFO1,True,self.fontcolor)
            ginfo2 = self.infofont.render(INFO2,True,self.fontcolor)
            self.image.blit(ginfo0,(self.rect.width/15,self.rect.height - 3*ginfo0.get_height() - 2*self.rect.height/40))
            self.image.blit(ginfo1,(self.rect.width/15,self.rect.height - 2*ginfo1.get_height() - 2*self.rect.height/40))
            self.image.blit(ginfo2,(self.rect.width/15,self.rect.height - ginfo2.get_height() - self.rect.height/20))
            
            # Draw Menu
            for i in range(0,len(self.options)):
                self.drawMenuItem(i,False)
            self.drawMenuItem(self.selected,True)
                
            
        
        # Draw our angry passenger here
        self.anim.setCustomScale(8.0)
        self.anim.update()
        self.anim.image.set_colorkey(Color('Red'))
        self.image.blit(self.anim.image,(self.rect.width/2,self.rect.height/10))

