# Elevate
# Copyright 2007 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

import pygame
from screen import *
from data import *

PANEL_FONT_NAME = "CAB.TTF" #"CAFENB__.TTF"

TGT_FLOOR_HDR = 'Destinations: '

def drawUpArrow(surface,color,prect,width=0):
	plist = []
	plist.append(prect.bottomleft)
	plist.append(prect.midleft)
	plist.append(prect.midtop)
	plist.append(prect.midright)
	plist.append(prect.bottomright)
	plist.append(prect.center)

	pygame.draw.polygon(surface,color,plist,width)

def drawDownArrow(surface,color,prect,width=0):
	plist = []
	plist.append(prect.topleft)
	plist.append(prect.midleft)
	plist.append(prect.midbottom)
	plist.append(prect.midright)
	plist.append(prect.topright)
	plist.append(prect.center)

	pygame.draw.polygon(surface,color,plist,width)


class Panel(pygame.sprite.Sprite):
    
    def __init__(self, screen, width):
        pygame.sprite.Sprite.__init__(self)

        # Set up the fonts
        self.scorefont = pygame.font.Font(filepath(PANEL_FONT_NAME),30)
        self.dirfont = pygame.font.Font(filepath(PANEL_FONT_NAME),50)
        self.tgtfont = pygame.font.Font(filepath(PANEL_FONT_NAME),20)

        self.rect = Rect(screen.get_width()-width,0,width,screen.get_height())
		
        self.delivered = 0
        self.complaints = 0
        self.tgts = []
		
        self.dir = 'Idle'
        self.vel = 0
        
        self.color = Color('Black')
        
        self.level_title = 'None'

    def createSurface(self):
        # Simple surface creation
        self.image = pygame.Surface((self.rect.width,self.rect.height))
        self.image.convert()
        self.image.set_colorkey(Color('White'))
        self.image.fill(Color('White'))

    def setScore(self, delivered, complaints):
        self.delivered = delivered
        self.complaints = complaints
		
    def setElevator(self,dir,vel):
        self.dir = dir
        self.vel = vel

    def setTargets(self,targetfloors):
        self.tgts = targetfloors
	
    def setLevelTitle(self,value):
        self.level_title = value
	
    def update(self):
        self.createSurface()
        
        # Write up the targets
        if len(self.tgts) > 0:
            tgt_floorheader = self.tgtfont.render(TGT_FLOOR_HDR,1,self.color)
            self.image.blit(tgt_floorheader,(10,self.rect.height - int(2.1*tgt_floorheader.get_height())))
            base_x = int(1.1*tgt_floorheader.get_width()) + 10
            x = base_x
            y = self.rect.height - int(2.1*tgt_floorheader.get_height())
            count = 0
            for p in self.tgts:
                tgt_one = self.tgtfont.render(str(p)+'  ',1,self.color)
                self.image.blit(tgt_one,(x,y))
                count = count + 1
                x = x + tgt_one.get_width()
                if count >= 4:
                    x = base_x
                    y = y + int(1.05*tgt_one.get_height())
                    count = 0

        # Elevator status
        dirtext = self.dirfont.render(self.dir,1,self.color)
        tx = (self.rect.width-dirtext.get_width())/2
        ty = (self.rect.height-dirtext.get_height())/2
        self.image.blit(dirtext,(tx,ty))
		                
        # Write the elevator settings
        aw = int(0.33*self.rect.width)
        ah = int(0.33*aw)
        ax = self.rect.width/2 - aw/2
        ay1 = ty-int(dirtext.get_height()/1.5)
        ay2 = ty+dirtext.get_height()

        bau = 0
        bad = 0
        
        if self.vel < 0:
            bau = self.vel
        else:
            bad = self.vel

        # All Elevator arrows
        for x in range(0,self.vel):
            drect = Rect(ax,ay2+x*ah,aw,ah)
            drawDownArrow(self.image,getRGBScaling(x*20+20),drect,0)
       
        for x in range(bad,5):
            drect = Rect(ax,ay2+x*ah,aw,ah)
            drawDownArrow(self.image,Color('Black'),drect,1)
                     
        for x in range(0,self.vel,-1):
            urect = Rect(ax,ay1+x*ah,aw,ah)
            drawUpArrow(self.image,getRGBScaling(-x*20+20),urect,0)
    
        for x in range(bau,-5,-1):
            urect = Rect(ax,ay1+x*ah,aw,ah)
            drawUpArrow(self.image,Color('Black'),urect,1)

        # The level title
        ltitle = self.tgtfont.render(self.level_title,1,self.color)
        self.image.blit(ltitle,((self.rect.width - ltitle.get_width())/2,10))

        # Now draw the current scores
        dtext = self.scorefont.render('Delivered: '+str(self.delivered),1,Color('Black'))
        cindex = (100*self.complaints)/MAX_COMPLAINTS
        ctext = self.scorefont.render('Complaints: '+str(self.complaints),1,getRGBScaling(min(cindex,100)))

        self.image.blit(dtext,(10,25+ltitle.get_height()))
        self.image.blit(ctext,(10,25+dtext.get_height()+ltitle.get_height()))
        
        

                         
        
		
